@@ -6,43 +6,49 @@
 
 #include "coredumpbackend.h"
 
+#include <chrono>
+
 #include <KCrash>
+
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+#include <QDBusMessage>
+#include <QDBusPendingCallWatcher>
+#include <QDBusReply>
 #include <QDebug>
+#include <QJsonDocument>
+#include <QJsonObject>
 #include <QProcess>
 #include <QScopeGuard>
-#include <QSettings>
 
 #include <unistd.h>
 
 // TODO: refactor the entire stack such that logs extraction happens in the processor and gets passed along the chain
 #include <systemd/sd-journal.h>
 
-#include <chrono>
-
 #include <coredump.h>
 
+#include "bugzillaintegration/reportinterface.h"
 #include "crashedapplication.h"
 #include "debugger.h"
 #include "debuggermanager.h"
 #include "drkonqi.h"
 #include "drkonqi_debug.h"
 #include "linuxprocmapsparser.h"
+#include <coredumpexcavator.h>
 
 using namespace std::chrono_literals;
 using namespace Qt::StringLiterals;
 
-// Only use signal safe API here.
-//   man 7 signal-safety
-static void emergencySaveFunction(int signal)
-{
-    // Should we crash while dealing with this crash, then make sure to remove the metadata file.
-    // Otherwise the helper daemon will call us again, and again cause a crash, until the user manually removes the file.
-    Q_UNUSED(signal);
-    unlink(qPrintable(CoredumpBackend::metadataPath()));
-}
+using namespace std::chrono_literals;
+using namespace Qt::StringLiterals;
 
 namespace
 {
+// Special backend type for core-based debugging. A polkit helper excavates the core file for us and we then
+// invoke gdb on it. Side stepping coredumpctl.
+constexpr auto CORE_BACKEND_TYPE = "coredump-core"_L1;
+
 [[nodiscard]] QString errnoError(const QString &msg, int err)
 {
     return msg + u": (%1) "_s.arg(QString::number(err)) + QString::fromLocal8Bit(strerror(err));
@@ -135,17 +141,18 @@ QList<EntriesHash> collectLogs(const QByteArray &cursor, sd_journal *context, co
 
 bool CoredumpBackend::init()
 {
-    KCrash::setEmergencySaveFunction(emergencySaveFunction);
-
     Q_ASSERT(!metadataPath().isEmpty());
     Q_ASSERT_X(QFile::exists(metadataPath()), static_cast<const char *>(Q_FUNC_INFO), qUtf8Printable(metadataPath()));
     qCDebug(DRKONQI_LOG) << "loading metadata" << metadataPath();
 
-    QSettings metadata(metadataPath(), QSettings::IniFormat);
-    metadata.beginGroup(QStringLiteral("Journal"));
-    const QStringList keys = metadata.allKeys();
-    for (const auto &key : keys) {
-        m_journalEntry.insert(key.toUtf8(), metadata.value(key).toByteArray());
+    QFile file(metadataPath());
+    if (!file.open(QFile::ReadOnly)) {
+        return false;
+    }
+    auto document = QJsonDocument::fromJson(file.readAll());
+    const auto journal = document[u"journal"_s].toObject().toVariantHash();
+    for (auto [key, value] : journal.asKeyValueRange()) {
+        m_journalEntry.insert(key.toUtf8(), value.toByteArray());
     }
     // conceivably the file contains no Journal group for unknown reasons
     Q_ASSERT_X(!m_journalEntry.isEmpty(), static_cast<const char *>(Q_FUNC_INFO), qUtf8Printable(metadataPath()));
@@ -157,6 +164,21 @@ bool CoredumpBackend::init()
         return false;
     }
 
+    connect(ReportInterface::self(), &ReportInterface::crashEventSent, this, [] {
+        QFile file(metadataPath());
+        if (!file.open(QFile::ReadWrite)) {
+            qCWarning(DRKONQI_LOG) << "Failed to open for writing" << metadataPath();
+            return;
+        }
+        auto object = QJsonDocument::fromJson(file.readAll()).object();
+        constexpr auto DRKONQI_KEY = "drkonqi"_L1;
+        auto drkonqiObject = object[DRKONQI_KEY].toObject();
+        drkonqiObject[u"sentryEventId"_s] = ReportInterface::self()->sentryEventId();
+        object[DRKONQI_KEY] = drkonqiObject;
+        file.reset();
+        file.write(QJsonDocument(object).toJson());
+    });
+
     return true;
 }
 
@@ -173,7 +195,7 @@ CrashedApplication *CoredumpBackend::constructCrashedApplication()
     const QFileInfo executable(QString::fromUtf8(m_journalEntry["COREDUMP_EXE"]));
     const int signal = m_journalEntry["COREDUMP_SIGNAL"].toInt(&ok);
     Q_ASSERT(ok);
-    const bool hasDeletedFiles = LinuxProc::hasMapsDeletedFiles(executable.path(), m_journalEntry["COREDUMP_PROC_MAPS"], LinuxProc::Check::Stat);
+    const bool hasDeletedFiles = LinuxProc::hasMapsDeletedFiles(executable.filePath(), m_journalEntry["COREDUMP_PROC_MAPS"], LinuxProc::Check::Stat);
 
     Q_ASSERT_X(m_journalEntry["COREDUMP_PID"].toInt() == DrKonqi::pid(),
                static_cast<const char *>(Q_FUNC_INFO),
@@ -213,8 +235,8 @@ CrashedApplication *CoredumpBackend::constructCrashedApplication()
 
 DebuggerManager *CoredumpBackend::constructDebuggerManager()
 {
-    const QList<Debugger> internalDebuggers = Debugger::availableInternalDebuggers(m_backendType);
-    const QList<Debugger> externalDebuggers = Debugger::availableExternalDebuggers(m_backendType);
+    const QList<Debugger> internalDebuggers = Debugger::availableInternalDebuggers(CORE_BACKEND_TYPE);
+    const QList<Debugger> externalDebuggers = Debugger::availableExternalDebuggers(CORE_BACKEND_TYPE);
 
     const Debugger preferredDebugger(Debugger::findDebugger(internalDebuggers, QStringLiteral("gdb")));
     qCDebug(DRKONQI_LOG) << "Using debugger:" << preferredDebugger.codeName();
@@ -225,44 +247,20 @@ DebuggerManager *CoredumpBackend::constructDebuggerManager()
 
 void CoredumpBackend::prepareForDebugger()
 {
-    if (m_preparationProc) {
-        return; // Preparation in progress.
-    }
-
-    // Legacy coredumpd doesn't support debugger arguments. We'll have to actually extract the core and manually trace on it,
-    // somewhat meh. When Ubuntu 20.04 either goes EOL or is deemed irrelevant enough we can remove the entire preparation
-    // rigging (even in AbstractDrKonqiBackend).
-    const bool needPreparation = (m_backendType == QLatin1String("coredumpd"));
-    const bool alreadyPrepared = (m_coreDir != nullptr);
-
-    if (!needPreparation || alreadyPrepared) {
-        // Synthesize a signal.
-        QMetaObject::invokeMethod(this, &CoredumpBackend::preparedForDebugger, Qt::QueuedConnection);
+    if (m_excavator) {
+        m_excavator->excavateFrom(QString::fromUtf8(m_journalEntry["COREDUMP_FILENAME"]));
         return;
     }
 
-    m_coreDir = std::make_unique<QTemporaryDir>(QDir::tempPath() + QStringLiteral("/kcrash-core"));
-    Q_ASSERT(m_coreDir->isValid());
-
-    const QString coreFile = m_coreDir->filePath(QStringLiteral("core"));
-    m_preparationProc = std::make_unique<QProcess>();
-    m_preparationProc->setProcessChannelMode(QProcess::ForwardedChannels);
-    m_preparationProc->setProgram(QStringLiteral("coredumpctl"));
-    m_preparationProc->setArguments({QStringLiteral("--output"), coreFile, QStringLiteral("dump"), QString::number(m_crashedApplication->pid())});
-    QObject::connect(
-        m_preparationProc.get(),
-        &QProcess::finished,
-        this,
-        [this, coreFile](int exitCode, QProcess::ExitStatus exitStatus) {
-            qDebug() << "Coredumpd core dumping completed" << exitCode << exitStatus << coreFile;
-            // We dont really care if the dumping failed. The debugger will fail if the core file isn't there and on the UI
-            // side there's not much point differentiating as the user won't be able to file a bug in any case.
-            m_preparationProc = nullptr;
-            m_crashedApplication->m_coreFile = coreFile;
-            Q_EMIT preparedForDebugger();
-        },
-        Qt::QueuedConnection /* queue so we don't delete the object out from under it */);
-    m_preparationProc->start();
+    m_excavator = std::make_unique<AutomaticCoredumpExcavator>();
+    connect(m_excavator.get(), &AutomaticCoredumpExcavator::failed, this, [this] {
+        Q_EMIT failedToPrepare();
+    });
+    connect(m_excavator.get(), &AutomaticCoredumpExcavator::excavated, this, [this](const QString &corePath) {
+        m_crashedApplication->m_coreFile = corePath;
+        Q_EMIT preparedForDebugger();
+    });
+    m_excavator->excavateFrom(QString::fromUtf8(m_journalEntry["COREDUMP_FILENAME"]));
 }
 
 #include "moc_coredumpbackend.cpp"
